"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamoPagination = void 0;
const crypto_js_1 = require("crypto-js");
const util_dynamodb_1 = require("@aws-sdk/util-dynamodb");
const dynamo_pagination_request_1 = require("../dto/request/dynamo-pagination.request");
const error_1 = require("./error");
const dynamo_filter_request_1 = require("../dto/request/dynamo-filter.request");
const enums_1 = require("../enums/enums");
const class_validator_1 = require("class-validator");
const class_transformer_1 = require("class-transformer");
const dynamo_helper_1 = require("../helpers/dynamo.helper");
class DynamoPagination {
    constructor(dynamodb) {
        this.columns = [];
        this.placeholders = [];
        this.condition = '';
        this.dynamodb = dynamodb;
    }
    async pagination(options, params = null, logs = false) {
        //Se agrega el filtro de correo (temporal)
        if (params == null) {
            params = new dynamo_pagination_request_1.DynamoPaginationRequest();
        }
        (0, class_validator_1.validate)(options);
        (0, class_validator_1.validate)(params);
        if (logs) {
            console.log("Params:", JSON.stringify(params, null, 4));
        }
        //Si hay from desencriptamos
        var from = null;
        if (params.from) {
            try {
                var fromDecoded = decodeURIComponent(params.from);
                var fromDecrypted = crypto_js_1.AES.decrypt(fromDecoded, options.secret).toString(crypto_js_1.enc.Utf8);
                from = JSON.parse(fromDecrypted);
            }
            catch (e) {
                throw new error_1.DevsStudioDynamoError(400, "invalid next key");
            }
        }
        //Definiciones
        for (let [column, definition] of Object.entries(options.definitions)) {
            if (!definition.hidden) {
                this.columns.push(column);
            }
        }
        this.condition = this.getCondition(options, params.filters);
        var statement = `SELECT "${this.columns.join('", "')}" FROM ${options.table}`;
        if (this.condition.trim().length > 0) {
            statement += ` WHERE ${this.condition}`;
        }
        var stsParams = {
            Statement: statement,
            Limit: params.limit,
            Parameters: this.placeholders.length > 0 ? Object.values((0, util_dynamodb_1.marshall)(this.placeholders)) : null,
            NextToken: (from === null || from === void 0 ? void 0 : from.current) || null,
        };
        if (logs) {
            console.log(stsParams);
        }
        try {
            const output = await this.dynamodb.executeStatement(stsParams);
            var results = {
                Items: (0, dynamo_helper_1.unmarshallAll)(output.Items),
                Prev: null,
                Next: null
            };
            if (from && (from === null || from === void 0 ? void 0 : from.current)) {
                // console.log("from", from);
                var prevEncrypted = crypto_js_1.AES.encrypt(JSON.stringify({
                    prev: null,
                    current: from.prev,
                }), options.secret).toString();
                results.Prev = encodeURIComponent(prevEncrypted);
            }
            //Si hay última key encriptamos
            if (output.NextToken) {
                // console.log("NextToken", results.NextToken);
                var nextEncrypted = crypto_js_1.AES.encrypt(JSON.stringify({
                    prev: from ? from.current : null,
                    current: output.NextToken,
                }), options.secret).toString();
                results.Next = encodeURIComponent(nextEncrypted);
            }
            return results;
        }
        catch (err) {
            if (err.message === 'NextToken does not match request') {
                return {
                    Items: [],
                    Prev: null,
                    Next: null
                };
            }
            else {
                throw err;
            }
        }
    }
    getCondition(options, filters) {
        var condition = '';
        for (let filter of filters) {
            filter = (0, class_transformer_1.plainToInstance)(dynamo_filter_request_1.DynamoFilterRequest, filter);
            //El atributo está definido
            var definition = options.definitions[filter.attr];
            if (filter.type === enums_1.DynamoFilterType.SUB || definition) {
                condition += this._setFilter(options, filter, condition);
            }
        }
        return condition;
    }
    _setFilter(options, filter, condition) {
        this._verifyFilterAttribute(options, filter);
        //Procesamos filtro
        return this._processFilter(options, filter, condition);
    }
    _verifyFilterAttribute(options, filter) {
        //Verificamos tipo
        if (filter.type === enums_1.DynamoFilterType.COLUMN) {
            if (!options.definitions[filter.val]) {
                throw new error_1.DevsStudioDynamoError(400, `attribute filter '${filter.val}' is not allowed`);
            }
        }
    }
    _processFilter(options, filter, condition) {
        switch (filter.type) {
            case enums_1.DynamoFilterType.SIMPLE:
                return this._processSimpleFilter(filter, condition);
            case enums_1.DynamoFilterType.COLUMN:
                return this._processColumnFilter(filter, condition);
            case enums_1.DynamoFilterType.BETWEEN:
                return this._processBetweenFilter(filter, false, condition);
            case enums_1.DynamoFilterType.NOT_BETWEEN:
                return this._processBetweenFilter(filter, true, condition);
            case enums_1.DynamoFilterType.IN:
                return this._processInFilter(filter, false, condition);
            case enums_1.DynamoFilterType.NOT_IN:
                return this._processInFilter(filter, true, condition);
            case enums_1.DynamoFilterType.IS:
                return this._processIsFilter(filter, false, condition);
            case enums_1.DynamoFilterType.NOT_IS:
                return this._processIsFilter(filter, true, condition);
            case enums_1.DynamoFilterType.SUB:
                return this._processSubFilter(options, filter, condition);
        }
    }
    _processSimpleFilter(filter, condition) {
        if (filter.opr === enums_1.DynamoFilterOperator.CONTAINS) {
            return (" " +
                this._getConn(filter.conn, condition) +
                " Contains(" +
                filter.attr +
                ", " +
                this._setPlaceholder(filter.val) +
                ")");
        }
        else {
            return (" " +
                this._getConn(filter.conn, condition) +
                " (" +
                filter.attr +
                " " +
                filter.opr +
                " " +
                this._setPlaceholder(filter.val) +
                ")");
        }
    }
    _processColumnFilter(filter, condition) {
        if (filter.opr === enums_1.DynamoFilterOperator.CONTAINS) {
            return (" " +
                this._getConn(filter.conn, condition) +
                " Contains(" +
                filter.attr +
                ", " +
                filter.val +
                ")");
        }
        else {
            return (" " +
                this._getConn(filter.conn, condition) +
                " (" +
                filter.attr +
                " " +
                filter.opr +
                " " +
                filter.val +
                ")");
        }
    }
    _processBetweenFilter(filter, not, condition) {
        //Hacemos split de los campos
        var vals = filter.val.split(",");
        //Debe tener dos valores siempre
        if (vals.length !== 2) {
            throw new error_1.DevsStudioDynamoError(400, `filter value should be an string with two elements separated by comma, when filter type is ${filter.type}`);
        }
        //Creamos
        if (not) {
            return (" " +
                this._getConn(filter.conn, condition) +
                " (" +
                filter.attr +
                " NOT BETWEEN " +
                this._setPlaceholder(vals[0]) +
                " AND " +
                this._setPlaceholder(vals[1]) +
                ")");
        }
        else {
            return (" " +
                this._getConn(filter.conn, condition) +
                " (" +
                filter.attr +
                " BETWEEN " +
                this._setPlaceholder(vals[0]) +
                " AND " +
                this._setPlaceholder(vals[1]) +
                ")");
        }
    }
    _processInFilter(filter, not, condition) {
        var vals = filter.val.split(",");
        var current_placeholders = [];
        //Cada elemento no debe ser array
        for (var j = 0; j < vals.length; j++) {
            current_placeholders.push(this._setPlaceholder(vals[j]));
        }
        //Creamos
        if (not) {
            return (" " +
                this._getConn(filter.conn, condition) +
                " (" +
                filter.attr +
                " NOT IN (" +
                current_placeholders.join(", ") +
                "))");
        }
        else {
            return (" " +
                this._getConn(filter.conn, condition) +
                " (" +
                filter.attr +
                " IN (" +
                current_placeholders.join(", ") +
                "))");
        }
    }
    _processIsFilter(filter, not, condition) {
        //Creamos
        if (not) {
            return (" " +
                this._getConn(filter.conn, condition) +
                " (" +
                filter.attr +
                " IS NOT " + filter.val + ")");
        }
        else {
            return (" " +
                this._getConn(filter.conn, condition) +
                " (" +
                filter.attr +
                " IS " + filter.val + ")");
        }
    }
    _processSubFilter(options, filter, condition) {
        return (" " +
            this._getConn(filter.conn, condition) +
            " (" +
            this.getCondition(options, filter.subfilters) +
            ")");
    }
    _getConn(conn, condition) {
        return condition.trim().length > 0 ? conn : "";
    }
    _setPlaceholder(value) {
        this.placeholders.push(value);
        return '?';
    }
    ;
}
exports.DynamoPagination = DynamoPagination;
