"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamoService = void 0;
const client_dynamodb_1 = require("@aws-sdk/client-dynamodb");
const util_dynamodb_1 = require("@aws-sdk/util-dynamodb");
const dynamo_pagination_1 = require("../classes/dynamo-pagination");
const dynamo_helper_1 = require("../helpers/dynamo.helper");
class DynamoService {
    constructor(config = null) {
        this.config = config;
        this.dynamoClient = new client_dynamodb_1.DynamoDBClient(config || {});
        this.dynamodb = new client_dynamodb_1.DynamoDB(config || {});
    }
    async getItem(table, key) {
        const getItemCommand = new client_dynamodb_1.GetItemCommand({
            TableName: table,
            Key: (0, util_dynamodb_1.marshall)(key),
        });
        const data = await this.dynamoClient.send(getItemCommand);
        return data.Item ? (0, dynamo_helper_1.unmarshallAndConvert)(data.Item) : null;
    }
    async putItem(table, item) {
        const putItemCommand = new client_dynamodb_1.PutItemCommand({
            TableName: table,
            Item: (0, util_dynamodb_1.marshall)(item, {
                convertClassInstanceToMap: true
            })
        });
        await this.dynamoClient.send(putItemCommand);
    }
    async deleteItem(table, key) {
        const deleteItemCommand = new client_dynamodb_1.DeleteItemCommand({
            TableName: table,
            Key: (0, util_dynamodb_1.marshall)(key),
        });
        await this.dynamoClient.send(deleteItemCommand);
    }
    async simpleUpdateItem(table, key, item) {
        var set = [];
        var values = {};
        var names = {};
        for (let [attribute, value] of Object.entries(item)) {
            set.push(`#${attribute} = :${attribute}`);
            values[`:${attribute}`] = value;
            names[`#${attribute}`] = attribute;
        }
        return await this.updateItem(table, key, 'SET ' + set.join(', '), values, names);
    }
    async updateItem(table, key, updateExpression, expressionAttributeValues = null, expressionAttributeNames = null) {
        const updateItemCommand = new client_dynamodb_1.UpdateItemCommand({
            TableName: table,
            Key: (0, util_dynamodb_1.marshall)(key),
            UpdateExpression: updateExpression,
            ExpressionAttributeValues: (0, util_dynamodb_1.marshall)(expressionAttributeValues, {
                convertClassInstanceToMap: true
            }),
            ExpressionAttributeNames: expressionAttributeNames,
        });
        return await this.dynamoClient.send(updateItemCommand);
    }
    async query(table, expression, conditionExpression, expressionValues = null) {
        var statement = `SELECT ${expression} FROM ${table}`;
        if (conditionExpression.trim().length > 0) {
            statement += ` WHERE ${conditionExpression}`;
        }
        var stsParams = {
            Statement: statement,
            Parameters: expressionValues.length > 0 ? Object.values((0, util_dynamodb_1.marshall)(expressionValues)) : null,
        };
        const output = await this.dynamodb.executeStatement(stsParams);
        return output;
    }
    async queryOne(table, expression, conditionExpression, expressionValues = null) {
        var output = await this.query(table, expression, conditionExpression, expressionValues);
        return output.Items.length > 0 ? (0, util_dynamodb_1.unmarshall)(output.Items[0]) : null;
    }
    async transactWriteItems(items) {
        var transactItems = [];
        for (let item of items) {
            transactItems.push({
                [item.type]: {
                    TableName: item.table,
                    Item: (0, util_dynamodb_1.marshall)(item.item, {
                        convertClassInstanceToMap: true
                    })
                }
            });
        }
        const transactWriteItemsCommand = new client_dynamodb_1.TransactWriteItemsCommand({
            TransactItems: transactItems
        });
        await this.dynamoClient.send(transactWriteItemsCommand);
    }
    // async getPaginated(options: PaginationOptions, params: PaginationParams): Promise<any> {
    //     const data = await pagination(this.dynamodb, options, params);
    //     return data;
    // }
    async getPaginated(options, params, logs = false) {
        var dynamoPagination = new dynamo_pagination_1.DynamoPagination(this.dynamodb);
        const data = await dynamoPagination.pagination(options, params, logs);
        return data;
    }
}
exports.DynamoService = DynamoService;
